/****************************************************************************
** Meta object code from reading C++ file 'QXmppOutgoingClient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppOutgoingClient.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppOutgoingClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QXmppOutgoingClientE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppOutgoingClient::qt_create_metaobjectdata<qt_meta_tag_ZN19QXmppOutgoingClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppOutgoingClient",
        "connected",
        "",
        "QXmpp::Private::SessionBegin",
        "disconnected",
        "QXmpp::Private::SessionEnd",
        "errorOccurred",
        "text",
        "QXmppOutgoingClient::ConnectionError",
        "details",
        "QXmppClient::Error",
        "oldError",
        "elementReceived",
        "QDomElement",
        "element",
        "bool&",
        "handled",
        "presenceReceived",
        "QXmppPresence",
        "messageReceived",
        "QXmppMessage",
        "iqReceived",
        "QXmppIq",
        "sslErrors",
        "QList<QSslError>",
        "errors"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void(const QXmpp::Private::SessionBegin &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void(const QXmpp::Private::SessionEnd &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &, const QXmppOutgoingClient::ConnectionError &, QXmppClient::Error)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'elementReceived'
        QtMocHelpers::SignalData<void(const QDomElement &, bool &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Signal 'presenceReceived'
        QtMocHelpers::SignalData<void(const QXmppPresence &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const QXmppMessage &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Signal 'iqReceived'
        QtMocHelpers::SignalData<void(const QXmppIq &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Signal 'sslErrors'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppOutgoingClient, qt_meta_tag_ZN19QXmppOutgoingClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppOutgoingClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppOutgoingClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppOutgoingClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QXmppOutgoingClientE_t>.metaTypes,
    nullptr
} };

void QXmppOutgoingClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppOutgoingClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected((*reinterpret_cast<std::add_pointer_t<QXmpp::Private::SessionBegin>>(_a[1]))); break;
        case 1: _t->disconnected((*reinterpret_cast<std::add_pointer_t<QXmpp::Private::SessionEnd>>(_a[1]))); break;
        case 2: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppOutgoingClient::ConnectionError>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QXmppClient::Error>>(_a[3]))); break;
        case 3: _t->elementReceived((*reinterpret_cast<std::add_pointer_t<QDomElement>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 4: _t->presenceReceived((*reinterpret_cast<std::add_pointer_t<QXmppPresence>>(_a[1]))); break;
        case 5: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[1]))); break;
        case 6: _t->iqReceived((*reinterpret_cast<std::add_pointer_t<QXmppIq>>(_a[1]))); break;
        case 7: _t->sslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QXmpp::Private::SessionBegin & )>(_a, &QXmppOutgoingClient::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QXmpp::Private::SessionEnd & )>(_a, &QXmppOutgoingClient::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QString & , const QXmppOutgoingClient::ConnectionError & , QXmppClient::Error )>(_a, &QXmppOutgoingClient::errorOccurred, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QDomElement & , bool & )>(_a, &QXmppOutgoingClient::elementReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QXmppPresence & )>(_a, &QXmppOutgoingClient::presenceReceived, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QXmppMessage & )>(_a, &QXmppOutgoingClient::messageReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QXmppIq & )>(_a, &QXmppOutgoingClient::iqReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOutgoingClient::*)(const QList<QSslError> & )>(_a, &QXmppOutgoingClient::sslErrors, 7))
            return;
    }
}

const QMetaObject *QXmppOutgoingClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppOutgoingClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppOutgoingClientE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppOutgoingClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QXmppOutgoingClient::connected(const QXmpp::Private::SessionBegin & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXmppOutgoingClient::disconnected(const QXmpp::Private::SessionEnd & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXmppOutgoingClient::errorOccurred(const QString & _t1, const QXmppOutgoingClient::ConnectionError & _t2, QXmppClient::Error _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QXmppOutgoingClient::elementReceived(const QDomElement & _t1, bool & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXmppOutgoingClient::presenceReceived(const QXmppPresence & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QXmppOutgoingClient::messageReceived(const QXmppMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QXmppOutgoingClient::iqReceived(const QXmppIq & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QXmppOutgoingClient::sslErrors(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
