/****************************************************************************
** Meta object code from reading C++ file 'QXmppClient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppClient.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QXmppClientE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppClient::qt_create_metaobjectdata<qt_meta_tag_ZN11QXmppClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppClient",
        "connected",
        "",
        "disconnected",
        "error",
        "QXmppClient::Error",
        "errorOccurred",
        "QXmppError",
        "loggerChanged",
        "QXmppLogger*",
        "logger",
        "messageReceived",
        "QXmppMessage",
        "message",
        "presenceReceived",
        "QXmppPresence",
        "presence",
        "iqReceived",
        "QXmppIq",
        "iq",
        "sslErrors",
        "QList<QSslError>",
        "errors",
        "stateChanged",
        "QXmppClient::State",
        "state",
        "credentialsChanged",
        "connectToServer",
        "QXmppConfiguration",
        "initialPresence",
        "jid",
        "password",
        "disconnectFromServer",
        "sendPacket",
        "QXmppNonza",
        "sendMessage",
        "bareJid",
        "State",
        "Error",
        "NoError",
        "SocketError",
        "KeepAliveError",
        "XmppStreamError",
        "DisconnectedState",
        "ConnectingState",
        "ConnectedState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QXmppClient::Error)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QXmppError &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'loggerChanged'
        QtMocHelpers::SignalData<void(QXmppLogger *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const QXmppMessage &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'presenceReceived'
        QtMocHelpers::SignalData<void(const QXmppPresence &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'iqReceived'
        QtMocHelpers::SignalData<void(const QXmppIq &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'sslErrors'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QXmppClient::State)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'credentialsChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void(const QXmppConfiguration &, const QXmppPresence &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 2 }, { 0x80000000 | 15, 29 },
        }}),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void(const QXmppConfiguration &)>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 28, 2 },
        }}),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::QString, 31 },
        }}),
        // Slot 'disconnectFromServer'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendPacket'
        QtMocHelpers::SlotData<bool(const QXmppNonza &)>(33, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 34, 2 },
        }}),
        // Slot 'sendMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::QString, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'logger'
        QtMocHelpers::PropertyData<QXmppLogger*>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(25, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(38, 38, QMC::EnumFlags{}).add({
            {   39, Error::NoError },
            {   40, Error::SocketError },
            {   41, Error::KeepAliveError },
            {   42, Error::XmppStreamError },
        }),
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(37, 37, QMC::EnumFlags{}).add({
            {   43, State::DisconnectedState },
            {   44, State::ConnectingState },
            {   45, State::ConnectedState },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppClient, qt_meta_tag_ZN11QXmppClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QXmppClientE_t>.metaTypes,
    nullptr
} };

void QXmppClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->error((*reinterpret_cast<std::add_pointer_t<QXmppClient::Error>>(_a[1]))); break;
        case 3: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QXmppError>>(_a[1]))); break;
        case 4: _t->loggerChanged((*reinterpret_cast<std::add_pointer_t<QXmppLogger*>>(_a[1]))); break;
        case 5: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[1]))); break;
        case 6: _t->presenceReceived((*reinterpret_cast<std::add_pointer_t<QXmppPresence>>(_a[1]))); break;
        case 7: _t->iqReceived((*reinterpret_cast<std::add_pointer_t<QXmppIq>>(_a[1]))); break;
        case 8: _t->sslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 9: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QXmppClient::State>>(_a[1]))); break;
        case 10: _t->credentialsChanged(); break;
        case 11: _t->connectToServer((*reinterpret_cast<std::add_pointer_t<QXmppConfiguration>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppPresence>>(_a[2]))); break;
        case 12: _t->connectToServer((*reinterpret_cast<std::add_pointer_t<QXmppConfiguration>>(_a[1]))); break;
        case 13: _t->connectToServer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->disconnectFromServer(); break;
        case 15: { bool _r = _t->sendPacket((*reinterpret_cast<std::add_pointer_t<QXmppNonza>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->sendMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppLogger* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)()>(_a, &QXmppClient::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)()>(_a, &QXmppClient::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(QXmppClient::Error )>(_a, &QXmppClient::error, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(const QXmppError & )>(_a, &QXmppClient::errorOccurred, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(QXmppLogger * )>(_a, &QXmppClient::loggerChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(const QXmppMessage & )>(_a, &QXmppClient::messageReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(const QXmppPresence & )>(_a, &QXmppClient::presenceReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(const QXmppIq & )>(_a, &QXmppClient::iqReceived, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(const QList<QSslError> & )>(_a, &QXmppClient::sslErrors, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)(QXmppClient::State )>(_a, &QXmppClient::stateChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppClient::*)()>(_a, &QXmppClient::credentialsChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QXmppLogger* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QXmppLogger**>(_v) = _t->logger(); break;
        case 1: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLogger(*reinterpret_cast<QXmppLogger**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppClientE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QXmppClient::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppClient::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmppClient::error(QXmppClient::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXmppClient::errorOccurred(const QXmppError & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QXmppClient::loggerChanged(QXmppLogger * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QXmppClient::messageReceived(const QXmppMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QXmppClient::presenceReceived(const QXmppPresence & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QXmppClient::iqReceived(const QXmppIq & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QXmppClient::sslErrors(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QXmppClient::stateChanged(QXmppClient::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QXmppClient::credentialsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
