/****************************************************************************
** Meta object code from reading C++ file 'QXmppMixManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppMixManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppMixManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QXmppMixManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppMixManager::qt_create_metaobjectdata<qt_meta_tag_ZN15QXmppMixManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppMixManager",
        "participantSupportChanged",
        "",
        "messageArchivingSupportChanged",
        "servicesChanged",
        "channelConfigurationUpdated",
        "channelJid",
        "QXmppMixConfigItem",
        "configuration",
        "channelInformationUpdated",
        "QXmppMixInfoItem",
        "information",
        "jidAllowed",
        "jid",
        "allJidsAllowed",
        "jidDisallowed",
        "allJidsDisallowed",
        "jidBanned",
        "jidUnbanned",
        "allJidsUnbanned",
        "participantReceived",
        "QXmppMixParticipantItem",
        "participant",
        "participantLeft",
        "participantId",
        "channelDeleted",
        "participantSupport",
        "Support",
        "messageArchivingSupport",
        "services",
        "QList<Service>",
        "Unknown",
        "Unsupported",
        "Supported"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'participantSupportChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'messageArchivingSupportChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'servicesChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'channelConfigurationUpdated'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMixConfigItem &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'channelInformationUpdated'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMixInfoItem &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'jidAllowed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'allJidsAllowed'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'jidDisallowed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'allJidsDisallowed'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'jidBanned'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'jidUnbanned'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'allJidsUnbanned'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'participantReceived'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMixParticipantItem &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 21, 22 },
        }}),
        // Signal 'participantLeft'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 24 },
        }}),
        // Signal 'channelDeleted'
        QtMocHelpers::SignalData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'participantSupport'
        QtMocHelpers::PropertyData<enum Support>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'messageArchivingSupport'
        QtMocHelpers::PropertyData<enum Support>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'services'
        QtMocHelpers::PropertyData<QList<Service>>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Support'
        QtMocHelpers::EnumData<enum Support>(27, 27, QMC::EnumIsScoped).add({
            {   31, Support::Unknown },
            {   32, Support::Unsupported },
            {   33, Support::Supported },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppMixManager, qt_meta_tag_ZN15QXmppMixManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppMixManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMixManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMixManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QXmppMixManagerE_t>.metaTypes,
    nullptr
} };

void QXmppMixManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppMixManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->participantSupportChanged(); break;
        case 1: _t->messageArchivingSupportChanged(); break;
        case 2: _t->servicesChanged(); break;
        case 3: _t->channelConfigurationUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMixConfigItem>>(_a[2]))); break;
        case 4: _t->channelInformationUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMixInfoItem>>(_a[2]))); break;
        case 5: _t->jidAllowed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->allJidsAllowed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->jidDisallowed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->allJidsDisallowed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->jidBanned((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->jidUnbanned((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->allJidsUnbanned((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->participantReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMixParticipantItem>>(_a[2]))); break;
        case 13: _t->participantLeft((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->channelDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMixConfigItem >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMixInfoItem >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMixParticipantItem >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)()>(_a, &QXmppMixManager::participantSupportChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)()>(_a, &QXmppMixManager::messageArchivingSupportChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)()>(_a, &QXmppMixManager::servicesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QXmppMixConfigItem & )>(_a, &QXmppMixManager::channelConfigurationUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QXmppMixInfoItem & )>(_a, &QXmppMixManager::channelInformationUpdated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QString & )>(_a, &QXmppMixManager::jidAllowed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & )>(_a, &QXmppMixManager::allJidsAllowed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QString & )>(_a, &QXmppMixManager::jidDisallowed, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & )>(_a, &QXmppMixManager::allJidsDisallowed, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QString & )>(_a, &QXmppMixManager::jidBanned, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QString & )>(_a, &QXmppMixManager::jidUnbanned, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & )>(_a, &QXmppMixManager::allJidsUnbanned, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QXmppMixParticipantItem & )>(_a, &QXmppMixManager::participantReceived, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & , const QString & )>(_a, &QXmppMixManager::participantLeft, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMixManager::*)(const QString & )>(_a, &QXmppMixManager::channelDeleted, 14))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Support*>(_v) = _t->participantSupport(); break;
        case 1: *reinterpret_cast<enum Support*>(_v) = _t->messageArchivingSupport(); break;
        case 2: *reinterpret_cast<QList<Service>*>(_v) = _t->services(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppMixManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppMixManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMixManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QXmppPubSubEventHandler"))
        return static_cast< QXmppPubSubEventHandler*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppMixManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QXmppMixManager::participantSupportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppMixManager::messageArchivingSupportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmppMixManager::servicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QXmppMixManager::channelConfigurationUpdated(const QString & _t1, const QXmppMixConfigItem & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXmppMixManager::channelInformationUpdated(const QString & _t1, const QXmppMixInfoItem & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QXmppMixManager::jidAllowed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QXmppMixManager::allJidsAllowed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QXmppMixManager::jidDisallowed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void QXmppMixManager::allJidsDisallowed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QXmppMixManager::jidBanned(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void QXmppMixManager::jidUnbanned(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void QXmppMixManager::allJidsUnbanned(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QXmppMixManager::participantReceived(const QString & _t1, const QXmppMixParticipantItem & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void QXmppMixManager::participantLeft(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void QXmppMixManager::channelDeleted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
QT_WARNING_POP
