/****************************************************************************
** Meta object code from reading C++ file 'QXmppOmemoManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qxmpp/src/omemo/QXmppOmemoManager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppOmemoManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QXmppOmemoManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppOmemoManager::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppOmemoManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppOmemoManager",
        "trustLevelsChanged",
        "",
        "QMultiHash<QString,QByteArray>",
        "modifiedKeys",
        "deviceAdded",
        "jid",
        "uint32_t",
        "deviceId",
        "deviceChanged",
        "deviceRemoved",
        "devicesRemoved",
        "allDevicesRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trustLevelsChanged'
        QtMocHelpers::SignalData<void(const QMultiHash<QString,QByteArray> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'deviceAdded'
        QtMocHelpers::SignalData<void(const QString &, uint32_t)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'deviceChanged'
        QtMocHelpers::SignalData<void(const QString &, uint32_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'deviceRemoved'
        QtMocHelpers::SignalData<void(const QString &, uint32_t)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'devicesRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'allDevicesRemoved'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppOmemoManager, qt_meta_tag_ZN17QXmppOmemoManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppOmemoManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppOmemoManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppOmemoManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppOmemoManagerE_t>.metaTypes,
    nullptr
} };

void QXmppOmemoManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppOmemoManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trustLevelsChanged((*reinterpret_cast<std::add_pointer_t<QMultiHash<QString,QByteArray>>>(_a[1]))); break;
        case 1: _t->deviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2]))); break;
        case 2: _t->deviceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2]))); break;
        case 3: _t->deviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2]))); break;
        case 4: _t->devicesRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->allDevicesRemoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)(const QMultiHash<QString,QByteArray> & )>(_a, &QXmppOmemoManager::trustLevelsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)(const QString & , uint32_t )>(_a, &QXmppOmemoManager::deviceAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)(const QString & , uint32_t )>(_a, &QXmppOmemoManager::deviceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)(const QString & , uint32_t )>(_a, &QXmppOmemoManager::deviceRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)(const QString & )>(_a, &QXmppOmemoManager::devicesRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppOmemoManager::*)()>(_a, &QXmppOmemoManager::allDevicesRemoved, 5))
            return;
    }
}

const QMetaObject *QXmppOmemoManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppOmemoManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppOmemoManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QXmppE2eeExtension"))
        return static_cast< QXmppE2eeExtension*>(this);
    if (!strcmp(_clname, "QXmppPubSubEventHandler"))
        return static_cast< QXmppPubSubEventHandler*>(this);
    if (!strcmp(_clname, "QXmppMessageHandler"))
        return static_cast< QXmppMessageHandler*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppOmemoManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QXmppOmemoManager::trustLevelsChanged(const QMultiHash<QString,QByteArray> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXmppOmemoManager::deviceAdded(const QString & _t1, uint32_t _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QXmppOmemoManager::deviceChanged(const QString & _t1, uint32_t _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QXmppOmemoManager::deviceRemoved(const QString & _t1, uint32_t _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXmppOmemoManager::devicesRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QXmppOmemoManager::allDevicesRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
