/****************************************************************************
** Meta object code from reading C++ file 'QXmppTransferManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppTransferManager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppTransferManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QXmppTransferJobE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppTransferJob::qt_create_metaobjectdata<qt_meta_tag_ZN16QXmppTransferJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppTransferJob",
        "error",
        "",
        "QXmppTransferJob::Error",
        "finished",
        "localFileUrlChanged",
        "QUrl",
        "localFileUrl",
        "progress",
        "done",
        "total",
        "stateChanged",
        "QXmppTransferJob::State",
        "state",
        "abort",
        "accept",
        "filePath",
        "QIODevice*",
        "output",
        "_q_terminated",
        "direction",
        "Direction",
        "jid",
        "method",
        "Method",
        "State",
        "fileName",
        "fileSize",
        "IncomingDirection",
        "OutgoingDirection",
        "Error",
        "NoError",
        "AbortError",
        "FileAccessError",
        "FileCorruptError",
        "ProtocolError",
        "NoMethod",
        "InBandMethod",
        "SocksMethod",
        "AnyMethod",
        "Methods",
        "OfferState",
        "StartState",
        "TransferState",
        "FinishedState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(QXmppTransferJob::Error)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 1 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localFileUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 9 }, { QMetaType::LongLong, 10 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QXmppTransferJob::State)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'accept'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void(QIODevice *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot '_q_terminated'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'direction'
        QtMocHelpers::PropertyData<enum Direction>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'localFileUrl'
        QtMocHelpers::PropertyData<QUrl>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'jid'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'method'
        QtMocHelpers::PropertyData<enum Method>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(13, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'fileName'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fileSize'
        QtMocHelpers::PropertyData<qint64>(27, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(21, 21, QMC::EnumFlags{}).add({
            {   28, Direction::IncomingDirection },
            {   29, Direction::OutgoingDirection },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(30, 30, QMC::EnumFlags{}).add({
            {   31, Error::NoError },
            {   32, Error::AbortError },
            {   33, Error::FileAccessError },
            {   34, Error::FileCorruptError },
            {   35, Error::ProtocolError },
        }),
        // enum 'Method'
        QtMocHelpers::EnumData<enum Method>(24, 24, QMC::EnumFlags{}).add({
            {   36, Method::NoMethod },
            {   37, Method::InBandMethod },
            {   38, Method::SocksMethod },
            {   39, Method::AnyMethod },
        }),
        // flag 'Methods'
        QtMocHelpers::EnumData<Methods>(40, 24, QMC::EnumIsFlag).add({
            {   36, Method::NoMethod },
            {   37, Method::InBandMethod },
            {   38, Method::SocksMethod },
            {   39, Method::AnyMethod },
        }),
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(25, 25, QMC::EnumFlags{}).add({
            {   41, State::OfferState },
            {   42, State::StartState },
            {   43, State::TransferState },
            {   44, State::FinishedState },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppTransferJob, qt_meta_tag_ZN16QXmppTransferJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppTransferJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QXmppTransferJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QXmppTransferJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QXmppTransferJobE_t>.metaTypes,
    nullptr
} };

void QXmppTransferJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppTransferJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob::Error>>(_a[1]))); break;
        case 1: _t->finished(); break;
        case 2: _t->localFileUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->progress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 4: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob::State>>(_a[1]))); break;
        case 5: _t->abort(); break;
        case 6: _t->accept((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->accept((*reinterpret_cast<std::add_pointer_t<QIODevice*>>(_a[1]))); break;
        case 8: _t->_q_terminated(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QIODevice* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferJob::*)(QXmppTransferJob::Error )>(_a, &QXmppTransferJob::error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferJob::*)()>(_a, &QXmppTransferJob::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferJob::*)(const QUrl & )>(_a, &QXmppTransferJob::localFileUrlChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferJob::*)(qint64 , qint64 )>(_a, &QXmppTransferJob::progress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferJob::*)(QXmppTransferJob::State )>(_a, &QXmppTransferJob::stateChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Direction*>(_v) = _t->direction(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->localFileUrl(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->jid(); break;
        case 3: *reinterpret_cast<enum Method*>(_v) = _t->method(); break;
        case 4: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->fileName(); break;
        case 6: *reinterpret_cast<qint64*>(_v) = _t->fileSize(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setLocalFileUrl(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppTransferJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppTransferJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QXmppTransferJobE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppTransferJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QXmppTransferJob::error(QXmppTransferJob::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXmppTransferJob::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmppTransferJob::localFileUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXmppTransferJob::progress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXmppTransferJob::stateChanged(QXmppTransferJob::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN20QXmppTransferManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppTransferManager::qt_create_metaobjectdata<qt_meta_tag_ZN20QXmppTransferManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppTransferManager",
        "fileReceived",
        "",
        "QXmppTransferJob*",
        "job",
        "jobStarted",
        "jobFinished",
        "sendFile",
        "jid",
        "filePath",
        "description",
        "QIODevice*",
        "device",
        "QXmppTransferFileInfo",
        "fileInfo",
        "sid",
        "_q_iqReceived",
        "QXmppIq",
        "_q_jobDestroyed",
        "object",
        "_q_jobError",
        "QXmppTransferJob::Error",
        "error",
        "_q_jobFinished",
        "_q_jobStateChanged",
        "QXmppTransferJob::State",
        "state",
        "_q_socksServerConnected",
        "QTcpSocket*",
        "socket",
        "hostName",
        "port",
        "proxy",
        "proxyOnly",
        "supportedMethods",
        "QXmppTransferJob::Methods"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileReceived'
        QtMocHelpers::SignalData<void(QXmppTransferJob *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'jobStarted'
        QtMocHelpers::SignalData<void(QXmppTransferJob *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'jobFinished'
        QtMocHelpers::SignalData<void(QXmppTransferJob *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'sendFile'
        QtMocHelpers::SlotData<QXmppTransferJob *(const QString &, const QString &, const QString &)>(7, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'sendFile'
        QtMocHelpers::SlotData<QXmppTransferJob *(const QString &, const QString &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'sendFile'
        QtMocHelpers::SlotData<QXmppTransferJob *(const QString &, QIODevice *, const QXmppTransferFileInfo &, const QString &)>(7, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'sendFile'
        QtMocHelpers::SlotData<QXmppTransferJob *(const QString &, QIODevice *, const QXmppTransferFileInfo &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot '_q_iqReceived'
        QtMocHelpers::SlotData<void(const QXmppIq &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Slot '_q_jobDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 19 },
        }}),
        // Slot '_q_jobError'
        QtMocHelpers::SlotData<void(QXmppTransferJob::Error)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot '_q_jobFinished'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_q_jobStateChanged'
        QtMocHelpers::SlotData<void(QXmppTransferJob::State)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot '_q_socksServerConnected'
        QtMocHelpers::SlotData<void(QTcpSocket *, const QString &, quint16)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 }, { QMetaType::QString, 30 }, { QMetaType::UShort, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'proxy'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'proxyOnly'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'supportedMethods'
        QtMocHelpers::PropertyData<QXmppTransferJob::Methods>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppTransferManager, qt_meta_tag_ZN20QXmppTransferManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN20QXmppTransferManagerE[] = {
    QMetaObject::SuperData::link<QXmppTransferJob::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QXmppTransferManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppTransferManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppTransferManagerE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN20QXmppTransferManagerE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QXmppTransferManagerE_t>.metaTypes,
    nullptr
} };

void QXmppTransferManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppTransferManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileReceived((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob*>>(_a[1]))); break;
        case 1: _t->jobStarted((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob*>>(_a[1]))); break;
        case 2: _t->jobFinished((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob*>>(_a[1]))); break;
        case 3: { QXmppTransferJob* _r = _t->sendFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QXmppTransferJob**>(_a[0]) = std::move(_r); }  break;
        case 4: { QXmppTransferJob* _r = _t->sendFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QXmppTransferJob**>(_a[0]) = std::move(_r); }  break;
        case 5: { QXmppTransferJob* _r = _t->sendFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QIODevice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QXmppTransferFileInfo>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QXmppTransferJob**>(_a[0]) = std::move(_r); }  break;
        case 6: { QXmppTransferJob* _r = _t->sendFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QIODevice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QXmppTransferFileInfo>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QXmppTransferJob**>(_a[0]) = std::move(_r); }  break;
        case 7: _t->_q_iqReceived((*reinterpret_cast<std::add_pointer_t<QXmppIq>>(_a[1]))); break;
        case 8: _t->_q_jobDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 9: _t->_q_jobError((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob::Error>>(_a[1]))); break;
        case 10: _t->_q_jobFinished(); break;
        case 11: _t->_q_jobStateChanged((*reinterpret_cast<std::add_pointer_t<QXmppTransferJob::State>>(_a[1]))); break;
        case 12: _t->_q_socksServerConnected((*reinterpret_cast<std::add_pointer_t<QTcpSocket*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint16>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppTransferJob* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppTransferJob* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppTransferJob* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QIODevice* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QIODevice* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferManager::*)(QXmppTransferJob * )>(_a, &QXmppTransferManager::fileReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferManager::*)(QXmppTransferJob * )>(_a, &QXmppTransferManager::jobStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTransferManager::*)(QXmppTransferJob * )>(_a, &QXmppTransferManager::jobFinished, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->proxy(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->proxyOnly(); break;
        case 2: *reinterpret_cast<QXmppTransferJob::Methods*>(_v) = _t->supportedMethods(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setProxy(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setProxyOnly(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setSupportedMethods(*reinterpret_cast<QXmppTransferJob::Methods*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppTransferManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppTransferManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppTransferManagerE_t>.strings))
        return static_cast<void*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppTransferManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QXmppTransferManager::fileReceived(QXmppTransferJob * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXmppTransferManager::jobStarted(QXmppTransferJob * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXmppTransferManager::jobFinished(QXmppTransferJob * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
