/****************************************************************************
** Meta object code from reading C++ file 'QXmppStun_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/base/QXmppStun_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppStun_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QXmppStunTransactionE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppStunTransaction::qt_create_metaobjectdata<qt_meta_tag_ZN20QXmppStunTransactionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppStunTransaction",
        "finished",
        "",
        "writeStun",
        "QXmppStunMessage",
        "request",
        "readStun",
        "response",
        "retry"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'writeStun'
        QtMocHelpers::SignalData<void(const QXmppStunMessage &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'readStun'
        QtMocHelpers::SlotData<void(const QXmppStunMessage &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 7 },
        }}),
        // Slot 'retry'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppStunTransaction, qt_meta_tag_ZN20QXmppStunTransactionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppStunTransaction::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppStunTransactionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppStunTransactionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QXmppStunTransactionE_t>.metaTypes,
    nullptr
} };

void QXmppStunTransaction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppStunTransaction *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->writeStun((*reinterpret_cast<std::add_pointer_t<QXmppStunMessage>>(_a[1]))); break;
        case 2: _t->readStun((*reinterpret_cast<std::add_pointer_t<QXmppStunMessage>>(_a[1]))); break;
        case 3: _t->retry(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppStunTransaction::*)()>(_a, &QXmppStunTransaction::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppStunTransaction::*)(const QXmppStunMessage & )>(_a, &QXmppStunTransaction::writeStun, 1))
            return;
    }
}

const QMetaObject *QXmppStunTransaction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppStunTransaction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppStunTransactionE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppStunTransaction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QXmppStunTransaction::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppStunTransaction::writeStun(const QXmppStunMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN17QXmppIceTransportE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppIceTransport::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppIceTransportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppIceTransport",
        "datagramReceived",
        "",
        "data",
        "QHostAddress",
        "host",
        "port",
        "disconnectFromHost"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'datagramReceived'
        QtMocHelpers::SignalData<void(const QByteArray &, const QHostAddress &, quint16)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::UShort, 6 },
        }}),
        // Slot 'disconnectFromHost'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppIceTransport, qt_meta_tag_ZN17QXmppIceTransportE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppIceTransport::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceTransportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceTransportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppIceTransportE_t>.metaTypes,
    nullptr
} };

void QXmppIceTransport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppIceTransport *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->datagramReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint16>>(_a[3]))); break;
        case 1: _t->disconnectFromHost(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppIceTransport::*)(const QByteArray & , const QHostAddress & , quint16 )>(_a, &QXmppIceTransport::datagramReceived, 0))
            return;
    }
}

const QMetaObject *QXmppIceTransport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppIceTransport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceTransportE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppIceTransport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QXmppIceTransport::datagramReceived(const QByteArray & _t1, const QHostAddress & _t2, quint16 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN19QXmppTurnAllocationE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppTurnAllocation::qt_create_metaobjectdata<qt_meta_tag_ZN19QXmppTurnAllocationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppTurnAllocation",
        "connected",
        "",
        "disconnected",
        "connectToHost",
        "disconnectFromHost",
        "readyRead",
        "refresh",
        "refreshChannels"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToHost'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'disconnectFromHost'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readyRead'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refreshChannels'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppTurnAllocation, qt_meta_tag_ZN19QXmppTurnAllocationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppTurnAllocation::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppIceTransport::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppTurnAllocationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppTurnAllocationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QXmppTurnAllocationE_t>.metaTypes,
    nullptr
} };

void QXmppTurnAllocation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppTurnAllocation *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->connectToHost(); break;
        case 3: _t->disconnectFromHost(); break;
        case 4: _t->readyRead(); break;
        case 5: _t->refresh(); break;
        case 6: _t->refreshChannels(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppTurnAllocation::*)()>(_a, &QXmppTurnAllocation::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppTurnAllocation::*)()>(_a, &QXmppTurnAllocation::disconnected, 1))
            return;
    }
}

const QMetaObject *QXmppTurnAllocation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppTurnAllocation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QXmppTurnAllocationE_t>.strings))
        return static_cast<void*>(this);
    return QXmppIceTransport::qt_metacast(_clname);
}

int QXmppTurnAllocation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppIceTransport::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QXmppTurnAllocation::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppTurnAllocation::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN17QXmppUdpTransportE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppUdpTransport::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppUdpTransportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppUdpTransport",
        "disconnectFromHost",
        "",
        "readyRead"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'disconnectFromHost'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readyRead'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppUdpTransport, qt_meta_tag_ZN17QXmppUdpTransportE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppUdpTransport::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppIceTransport::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppUdpTransportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppUdpTransportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppUdpTransportE_t>.metaTypes,
    nullptr
} };

void QXmppUdpTransport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppUdpTransport *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->disconnectFromHost(); break;
        case 1: _t->readyRead(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *QXmppUdpTransport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppUdpTransport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppUdpTransportE_t>.strings))
        return static_cast<void*>(this);
    return QXmppIceTransport::qt_metacast(_clname);
}

int QXmppUdpTransport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppIceTransport::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
