/****************************************************************************
** Meta object code from reading C++ file 'QXmppJingleMessageInitiationManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppJingleMessageInitiationManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppJingleMessageInitiationManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppJingleMessageInitiation::qt_create_metaobjectdata<qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppJingleMessageInitiation",
        "ringing",
        "",
        "proceeded",
        "id",
        "remoteResource",
        "closed",
        "Result",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ringing'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'proceeded'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'closed'
        QtMocHelpers::SignalData<void(const Result &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppJingleMessageInitiation, qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppJingleMessageInitiation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>.metaTypes,
    nullptr
} };

void QXmppJingleMessageInitiation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppJingleMessageInitiation *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ringing(); break;
        case 1: _t->proceeded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->closed((*reinterpret_cast<std::add_pointer_t<Result>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppJingleMessageInitiation::*)()>(_a, &QXmppJingleMessageInitiation::ringing, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppJingleMessageInitiation::*)(const QString & , const QString & )>(_a, &QXmppJingleMessageInitiation::proceeded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppJingleMessageInitiation::*)(const Result & )>(_a, &QXmppJingleMessageInitiation::closed, 2))
            return;
    }
}

const QMetaObject *QXmppJingleMessageInitiation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppJingleMessageInitiation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QXmppJingleMessageInitiationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QXmppJingleMessageInitiation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QXmppJingleMessageInitiation::ringing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppJingleMessageInitiation::proceeded(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QXmppJingleMessageInitiation::closed(const Result & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppJingleMessageInitiationManager::qt_create_metaobjectdata<qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppJingleMessageInitiationManager",
        "proposed",
        "",
        "std::shared_ptr<QXmppJingleMessageInitiation>",
        "jmi",
        "id",
        "std::optional<QXmppJingleRtpDescription>",
        "description",
        "proposeReceived",
        "QList<QXmppJingleRtpDescription>",
        "descriptions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'proposed'
        QtMocHelpers::SignalData<void(const std::shared_ptr<QXmppJingleMessageInitiation> &, const QString &, const std::optional<QXmppJingleRtpDescription> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'proposeReceived'
        QtMocHelpers::SignalData<void(const std::shared_ptr<QXmppJingleMessageInitiation> &, const QString &, const QList<QXmppJingleRtpDescription> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppJingleMessageInitiationManager, qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppJingleMessageInitiationManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>.metaTypes,
    nullptr
} };

void QXmppJingleMessageInitiationManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppJingleMessageInitiationManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->proposed((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<QXmppJingleMessageInitiation>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::optional<QXmppJingleRtpDescription>>>(_a[3]))); break;
        case 1: _t->proposeReceived((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<QXmppJingleMessageInitiation>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<QXmppJingleRtpDescription>>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppJingleMessageInitiationManager::*)(const std::shared_ptr<QXmppJingleMessageInitiation> & , const QString & , const std::optional<QXmppJingleRtpDescription> & )>(_a, &QXmppJingleMessageInitiationManager::proposed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppJingleMessageInitiationManager::*)(const std::shared_ptr<QXmppJingleMessageInitiation> & , const QString & , const QList<QXmppJingleRtpDescription> & )>(_a, &QXmppJingleMessageInitiationManager::proposeReceived, 1))
            return;
    }
}

const QMetaObject *QXmppJingleMessageInitiationManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppJingleMessageInitiationManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QXmppJingleMessageInitiationManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QXmppMessageHandler"))
        return static_cast< QXmppMessageHandler*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppJingleMessageInitiationManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QXmppJingleMessageInitiationManager::proposed(const std::shared_ptr<QXmppJingleMessageInitiation> & _t1, const QString & _t2, const std::optional<QXmppJingleRtpDescription> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void QXmppJingleMessageInitiationManager::proposeReceived(const std::shared_ptr<QXmppJingleMessageInitiation> & _t1, const QString & _t2, const QList<QXmppJingleRtpDescription> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
