/****************************************************************************
** Meta object code from reading C++ file 'QXmppMucManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppMucManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppMucManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QXmppMucManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppMucManager::qt_create_metaobjectdata<qt_meta_tag_ZN15QXmppMucManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppMucManager",
        "invitationReceived",
        "",
        "roomJid",
        "inviter",
        "reason",
        "roomAdded",
        "QXmppMucRoom*",
        "room",
        "_q_messageReceived",
        "QXmppMessage",
        "message",
        "_q_roomDestroyed",
        "object",
        "rooms",
        "QList<QXmppMucRoom*>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'invitationReceived'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'roomAdded'
        QtMocHelpers::SignalData<void(QXmppMucRoom *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot '_q_messageReceived'
        QtMocHelpers::SlotData<void(const QXmppMessage &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot '_q_roomDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rooms'
        QtMocHelpers::PropertyData<QList<QXmppMucRoom*>>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppMucManager, qt_meta_tag_ZN15QXmppMucManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppMucManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMucManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMucManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QXmppMucManagerE_t>.metaTypes,
    nullptr
} };

void QXmppMucManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppMucManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->invitationReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->roomAdded((*reinterpret_cast<std::add_pointer_t<QXmppMucRoom*>>(_a[1]))); break;
        case 2: _t->_q_messageReceived((*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[1]))); break;
        case 3: _t->_q_roomDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMucRoom* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManager::*)(const QString & , const QString & , const QString & )>(_a, &QXmppMucManager::invitationReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManager::*)(QXmppMucRoom * )>(_a, &QXmppMucManager::roomAdded, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QXmppMucRoom*> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QXmppMucRoom*>*>(_v) = _t->rooms(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppMucManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppMucManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppMucManagerE_t>.strings))
        return static_cast<void*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppMucManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QXmppMucManager::invitationReceived(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void QXmppMucManager::roomAdded(QXmppMucRoom * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN12QXmppMucRoomE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppMucRoom::qt_create_metaobjectdata<qt_meta_tag_ZN12QXmppMucRoomE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppMucRoom",
        "allowedActionsChanged",
        "",
        "QXmppMucRoom::Actions",
        "actions",
        "configurationReceived",
        "QXmppDataForm",
        "configuration",
        "error",
        "QXmppStanza::Error",
        "joined",
        "kicked",
        "jid",
        "reason",
        "isJoinedChanged",
        "left",
        "messageReceived",
        "QXmppMessage",
        "message",
        "nameChanged",
        "name",
        "nickNameChanged",
        "nickName",
        "participantAdded",
        "participantChanged",
        "participantRemoved",
        "participantsChanged",
        "permissionsReceived",
        "QList<QXmppMucItem>",
        "permissions",
        "subjectChanged",
        "subject",
        "ban",
        "join",
        "kick",
        "leave",
        "requestConfiguration",
        "requestPermissions",
        "setConfiguration",
        "form",
        "setPermissions",
        "sendInvitation",
        "sendMessage",
        "text",
        "_q_disconnected",
        "_q_messageReceived",
        "_q_presenceReceived",
        "QXmppPresence",
        "presence",
        "participantFullJid",
        "allowedActions",
        "isJoined",
        "participants",
        "password",
        "Action",
        "NoAction",
        "SubjectAction",
        "ConfigurationAction",
        "PermissionsAction",
        "KickAction",
        "Actions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'allowedActionsChanged'
        QtMocHelpers::SignalData<void(QXmppMucRoom::Actions)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'configurationReceived'
        QtMocHelpers::SignalData<void(const QXmppDataForm &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QXmppStanza::Error &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 8 },
        }}),
        // Signal 'joined'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'kicked'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'isJoinedChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'left'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const QXmppMessage &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'nickNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'participantAdded'
        QtMocHelpers::SignalData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'participantChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'participantRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'participantsChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'permissionsReceived'
        QtMocHelpers::SignalData<void(const QList<QXmppMucItem> &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'subjectChanged'
        QtMocHelpers::SignalData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'ban'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'join'
        QtMocHelpers::SlotData<bool()>(33, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'kick'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'leave'
        QtMocHelpers::SlotData<bool(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'leave'
        QtMocHelpers::SlotData<bool()>(35, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'requestConfiguration'
        QtMocHelpers::SlotData<bool()>(36, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'requestPermissions'
        QtMocHelpers::SlotData<bool()>(37, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setConfiguration'
        QtMocHelpers::SlotData<bool(const QXmppDataForm &)>(38, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 6, 39 },
        }}),
        // Slot 'setPermissions'
        QtMocHelpers::SlotData<bool(const QList<QXmppMucItem> &)>(40, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'sendInvitation'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'sendMessage'
        QtMocHelpers::SlotData<bool(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot '_q_disconnected'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_q_messageReceived'
        QtMocHelpers::SlotData<void(const QXmppMessage &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot '_q_presenceReceived'
        QtMocHelpers::SlotData<void(const QXmppPresence &)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Method 'participantFullJid'
        QtMocHelpers::MethodData<QString(const QString &) const>(49, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'allowedActions'
        QtMocHelpers::PropertyData<QXmppMucRoom::Actions>(50, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'isJoined'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'jid'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'nickName'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'participants'
        QtMocHelpers::PropertyData<QStringList>(52, QMetaType::QStringList, QMC::DefaultPropertyFlags, 13),
        // property 'password'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'subject'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Action'
        QtMocHelpers::EnumData<enum Action>(54, 54, QMC::EnumFlags{}).add({
            {   55, Action::NoAction },
            {   56, Action::SubjectAction },
            {   57, Action::ConfigurationAction },
            {   58, Action::PermissionsAction },
            {   59, Action::KickAction },
        }),
        // flag 'Actions'
        QtMocHelpers::EnumData<Actions>(60, 54, QMC::EnumIsFlag).add({
            {   55, Action::NoAction },
            {   56, Action::SubjectAction },
            {   57, Action::ConfigurationAction },
            {   58, Action::PermissionsAction },
            {   59, Action::KickAction },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppMucRoom, qt_meta_tag_ZN12QXmppMucRoomE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppMucRoom::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QXmppMucRoomE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QXmppMucRoomE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QXmppMucRoomE_t>.metaTypes,
    nullptr
} };

void QXmppMucRoom::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppMucRoom *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->allowedActionsChanged((*reinterpret_cast<std::add_pointer_t<QXmppMucRoom::Actions>>(_a[1]))); break;
        case 1: _t->configurationReceived((*reinterpret_cast<std::add_pointer_t<QXmppDataForm>>(_a[1]))); break;
        case 2: _t->error((*reinterpret_cast<std::add_pointer_t<QXmppStanza::Error>>(_a[1]))); break;
        case 3: _t->joined(); break;
        case 4: _t->kicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->isJoinedChanged(); break;
        case 6: _t->left(); break;
        case 7: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[1]))); break;
        case 8: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->nickNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->participantAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->participantChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->participantRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->participantsChanged(); break;
        case 14: _t->permissionsReceived((*reinterpret_cast<std::add_pointer_t<QList<QXmppMucItem>>>(_a[1]))); break;
        case 15: _t->subjectChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: { bool _r = _t->ban((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->join();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->kick((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->leave((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->leave();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->requestConfiguration();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->requestPermissions();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->setConfiguration((*reinterpret_cast<std::add_pointer_t<QXmppDataForm>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->setPermissions((*reinterpret_cast<std::add_pointer_t<QList<QXmppMucItem>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->sendInvitation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->sendMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->_q_disconnected(); break;
        case 28: _t->_q_messageReceived((*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[1]))); break;
        case 29: _t->_q_presenceReceived((*reinterpret_cast<std::add_pointer_t<QXmppPresence>>(_a[1]))); break;
        case 30: { QString _r = _t->participantFullJid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppDataForm >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppDataForm >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(QXmppMucRoom::Actions )>(_a, &QXmppMucRoom::allowedActionsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QXmppDataForm & )>(_a, &QXmppMucRoom::configurationReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QXmppStanza::Error & )>(_a, &QXmppMucRoom::error, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)()>(_a, &QXmppMucRoom::joined, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & , const QString & )>(_a, &QXmppMucRoom::kicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)()>(_a, &QXmppMucRoom::isJoinedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)()>(_a, &QXmppMucRoom::left, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QXmppMessage & )>(_a, &QXmppMucRoom::messageReceived, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::nameChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::nickNameChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::participantAdded, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::participantChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::participantRemoved, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)()>(_a, &QXmppMucRoom::participantsChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QList<QXmppMucItem> & )>(_a, &QXmppMucRoom::permissionsReceived, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucRoom::*)(const QString & )>(_a, &QXmppMucRoom::subjectChanged, 15))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QXmppMucRoom::Actions*>(_v) = _t->allowedActions(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isJoined(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->jid(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->nickName(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->participants(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->password(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->subject(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setNickName(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setPassword(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setSubject(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppMucRoom::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppMucRoom::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QXmppMucRoomE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QXmppMucRoom::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QXmppMucRoom::allowedActionsChanged(QXmppMucRoom::Actions _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXmppMucRoom::configurationReceived(const QXmppDataForm & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXmppMucRoom::error(const QXmppStanza::Error & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXmppMucRoom::joined()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QXmppMucRoom::kicked(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QXmppMucRoom::isJoinedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QXmppMucRoom::left()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QXmppMucRoom::messageReceived(const QXmppMessage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QXmppMucRoom::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QXmppMucRoom::nickNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QXmppMucRoom::participantAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QXmppMucRoom::participantChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QXmppMucRoom::participantRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QXmppMucRoom::participantsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QXmppMucRoom::permissionsReceived(const QList<QXmppMucItem> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QXmppMucRoom::subjectChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}
QT_WARNING_POP
