/****************************************************************************
** Meta object code from reading C++ file 'XmppSocket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/base/XmppSocket.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'XmppSocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmpp::Private::XmppSocket::qt_create_metaobjectdata<qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmpp::Private::XmppSocket",
        "started",
        "",
        "disconnected",
        "stanzaReceived",
        "QDomElement",
        "streamReceived",
        "QXmpp::Private::StreamOpen",
        "streamClosed",
        "errorOccurred",
        "text",
        "std::variant<StreamError,QAbstractSocket::SocketError>",
        "condition",
        "sslErrorsOccurred",
        "QList<QSslError>",
        "errors",
        "internalSocketStateChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stanzaReceived'
        QtMocHelpers::SignalData<void(const QDomElement &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'streamReceived'
        QtMocHelpers::SignalData<void(const QXmpp::Private::StreamOpen &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'streamClosed'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &, std::variant<StreamError,QAbstractSocket::SocketError>)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'sslErrorsOccurred'
        QtMocHelpers::SignalData<void(const QList<QSslError> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'internalSocketStateChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<XmppSocket, qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmpp::Private::XmppSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>.metaTypes,
    nullptr
} };

void QXmpp::Private::XmppSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XmppSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->stanzaReceived((*reinterpret_cast<std::add_pointer_t<QDomElement>>(_a[1]))); break;
        case 3: _t->streamReceived((*reinterpret_cast<std::add_pointer_t<QXmpp::Private::StreamOpen>>(_a[1]))); break;
        case 4: _t->streamClosed(); break;
        case 5: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::variant<StreamError,QAbstractSocket::SocketError>>>(_a[2]))); break;
        case 6: _t->sslErrorsOccurred((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 7: _t->internalSocketStateChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)()>(_a, &XmppSocket::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)()>(_a, &XmppSocket::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)(const QDomElement & )>(_a, &XmppSocket::stanzaReceived, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)(const QXmpp::Private::StreamOpen & )>(_a, &XmppSocket::streamReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)()>(_a, &XmppSocket::streamClosed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)(const QString & , std::variant<StreamError,QAbstractSocket::SocketError> )>(_a, &XmppSocket::errorOccurred, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)(const QList<QSslError> & )>(_a, &XmppSocket::sslErrorsOccurred, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (XmppSocket::*)()>(_a, &XmppSocket::internalSocketStateChanged, 7))
            return;
    }
}

const QMetaObject *QXmpp::Private::XmppSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmpp::Private::XmppSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5QXmpp7Private10XmppSocketE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SendDataInterface"))
        return static_cast< SendDataInterface*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmpp::Private::XmppSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QXmpp::Private::XmppSocket::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmpp::Private::XmppSocket::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmpp::Private::XmppSocket::stanzaReceived(const QDomElement & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXmpp::Private::XmppSocket::streamReceived(const QXmpp::Private::StreamOpen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QXmpp::Private::XmppSocket::streamClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QXmpp::Private::XmppSocket::errorOccurred(const QString & _t1, std::variant<StreamError,QAbstractSocket::SocketError> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QXmpp::Private::XmppSocket::sslErrorsOccurred(const QList<QSslError> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QXmpp::Private::XmppSocket::internalSocketStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
