/****************************************************************************
** Meta object code from reading C++ file 'QXmppHttpUploadManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppHttpUploadManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppHttpUploadManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QXmppHttpUploadE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppHttpUpload::qt_create_metaobjectdata<qt_meta_tag_ZN15QXmppHttpUploadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppHttpUpload",
        "progressChanged",
        "",
        "finished",
        "QXmppHttpUpload::Result",
        "result",
        "progress",
        "bytesSent",
        "bytesTotal"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'progressChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(const QXmppHttpUpload::Result &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'progress'
        QtMocHelpers::PropertyData<float>(6, QMetaType::Float, QMC::DefaultPropertyFlags, 0),
        // property 'bytesSent'
        QtMocHelpers::PropertyData<quint64>(7, QMetaType::ULongLong, QMC::DefaultPropertyFlags, 0),
        // property 'bytesTotal'
        QtMocHelpers::PropertyData<quint64>(8, QMetaType::ULongLong, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppHttpUpload, qt_meta_tag_ZN15QXmppHttpUploadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppHttpUpload::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppHttpUploadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppHttpUploadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QXmppHttpUploadE_t>.metaTypes,
    nullptr
} };

void QXmppHttpUpload::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppHttpUpload *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->progressChanged(); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<QXmppHttpUpload::Result>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppHttpUpload::Result >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppHttpUpload::*)()>(_a, &QXmppHttpUpload::progressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppHttpUpload::*)(const QXmppHttpUpload::Result & )>(_a, &QXmppHttpUpload::finished, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<float*>(_v) = _t->progress(); break;
        case 1: *reinterpret_cast<quint64*>(_v) = _t->bytesSent(); break;
        case 2: *reinterpret_cast<quint64*>(_v) = _t->bytesTotal(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppHttpUpload::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppHttpUpload::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppHttpUploadE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QXmppHttpUpload::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QXmppHttpUpload::progressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppHttpUpload::finished(const QXmppHttpUpload::Result & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN22QXmppHttpUploadManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppHttpUploadManager::qt_create_metaobjectdata<qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppHttpUploadManager",
        "servicesChanged",
        "",
        "supportChanged",
        "services",
        "QList<QXmppHttpUploadService>",
        "support",
        "QXmppHttpUploadManager::Support",
        "Support",
        "Unknown",
        "Unsupported",
        "Supported"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'servicesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'services'
        QtMocHelpers::PropertyData<QList<QXmppHttpUploadService>>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'support'
        QtMocHelpers::PropertyData<QXmppHttpUploadManager::Support>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Support'
        QtMocHelpers::EnumData<enum Support>(8, 8, QMC::EnumIsScoped).add({
            {    9, Support::Unknown },
            {   10, Support::Unsupported },
            {   11, Support::Supported },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppHttpUploadManager, qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppHttpUploadManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>.metaTypes,
    nullptr
} };

void QXmppHttpUploadManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppHttpUploadManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->servicesChanged(); break;
        case 1: _t->supportChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppHttpUploadManager::*)()>(_a, &QXmppHttpUploadManager::servicesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppHttpUploadManager::*)()>(_a, &QXmppHttpUploadManager::supportChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QXmppHttpUploadService>*>(_v) = _t->services(); break;
        case 1: *reinterpret_cast<QXmppHttpUploadManager::Support*>(_v) = _t->support(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppHttpUploadManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppHttpUploadManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QXmppHttpUploadManagerE_t>.strings))
        return static_cast<void*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppHttpUploadManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QXmppHttpUploadManager::servicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppHttpUploadManager::supportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
