/****************************************************************************
** Meta object code from reading C++ file 'QXmppStun.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/base/QXmppStun.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppStun.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QXmppIceComponentE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppIceComponent::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppIceComponentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppIceComponent",
        "connected",
        "",
        "datagramReceived",
        "datagram",
        "gatheringStateChanged",
        "localCandidatesChanged",
        "close",
        "connectToHost",
        "sendDatagram",
        "checkCandidates",
        "handleDatagram",
        "QHostAddress",
        "host",
        "port",
        "turnConnected",
        "updateGatheringState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'datagramReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Signal 'gatheringStateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localCandidatesChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToHost'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDatagram'
        QtMocHelpers::SlotData<qint64(const QByteArray &)>(9, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QByteArray, 4 },
        }}),
        // Slot 'checkCandidates'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleDatagram'
        QtMocHelpers::SlotData<void(const QByteArray &, const QHostAddress &, quint16)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 4 }, { 0x80000000 | 12, 13 }, { QMetaType::UShort, 14 },
        }}),
        // Slot 'turnConnected'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateGatheringState'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppIceComponent, qt_meta_tag_ZN17QXmppIceComponentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppIceComponent::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceComponentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceComponentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppIceComponentE_t>.metaTypes,
    nullptr
} };

void QXmppIceComponent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppIceComponent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->datagramReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->gatheringStateChanged(); break;
        case 3: _t->localCandidatesChanged(); break;
        case 4: _t->close(); break;
        case 5: _t->connectToHost(); break;
        case 6: { qint64 _r = _t->sendDatagram((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->checkCandidates(); break;
        case 8: _t->handleDatagram((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint16>>(_a[3]))); break;
        case 9: _t->turnConnected(); break;
        case 10: _t->updateGatheringState(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppIceComponent::*)()>(_a, &QXmppIceComponent::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceComponent::*)(const QByteArray & )>(_a, &QXmppIceComponent::datagramReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceComponent::*)()>(_a, &QXmppIceComponent::gatheringStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceComponent::*)()>(_a, &QXmppIceComponent::localCandidatesChanged, 3))
            return;
    }
}

const QMetaObject *QXmppIceComponent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppIceComponent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppIceComponentE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppIceComponent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void QXmppIceComponent::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppIceComponent::datagramReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXmppIceComponent::gatheringStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QXmppIceComponent::localCandidatesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN18QXmppIceConnectionE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppIceConnection::qt_create_metaobjectdata<qt_meta_tag_ZN18QXmppIceConnectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppIceConnection",
        "connected",
        "",
        "disconnected",
        "gatheringStateChanged",
        "localCandidatesChanged",
        "close",
        "connectToHost",
        "slotConnected",
        "slotGatheringStateChanged",
        "slotTimeout",
        "gatheringState",
        "QXmppIceConnection::GatheringState",
        "GatheringState",
        "NewGatheringState",
        "BusyGatheringState",
        "CompleteGatheringState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gatheringStateChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'localCandidatesChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToHost'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotConnected'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGatheringStateChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTimeout'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'gatheringState'
        QtMocHelpers::PropertyData<QXmppIceConnection::GatheringState>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GatheringState'
        QtMocHelpers::EnumData<enum GatheringState>(13, 13, QMC::EnumFlags{}).add({
            {   14, GatheringState::NewGatheringState },
            {   15, GatheringState::BusyGatheringState },
            {   16, GatheringState::CompleteGatheringState },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppIceConnection, qt_meta_tag_ZN18QXmppIceConnectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppIceConnection::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QXmppIceConnectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QXmppIceConnectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QXmppIceConnectionE_t>.metaTypes,
    nullptr
} };

void QXmppIceConnection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppIceConnection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->gatheringStateChanged(); break;
        case 3: _t->localCandidatesChanged(); break;
        case 4: _t->close(); break;
        case 5: _t->connectToHost(); break;
        case 6: _t->slotConnected(); break;
        case 7: _t->slotGatheringStateChanged(); break;
        case 8: _t->slotTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppIceConnection::*)()>(_a, &QXmppIceConnection::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceConnection::*)()>(_a, &QXmppIceConnection::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceConnection::*)()>(_a, &QXmppIceConnection::gatheringStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppIceConnection::*)()>(_a, &QXmppIceConnection::localCandidatesChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QXmppIceConnection::GatheringState*>(_v) = _t->gatheringState(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppIceConnection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppIceConnection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QXmppIceConnectionE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppIceConnection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QXmppIceConnection::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppIceConnection::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmppIceConnection::gatheringStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QXmppIceConnection::localCandidatesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
