/****************************************************************************
** Meta object code from reading C++ file 'QXmppRegistrationManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppRegistrationManager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppRegistrationManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24QXmppRegistrationManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppRegistrationManager::qt_create_metaobjectdata<qt_meta_tag_ZN24QXmppRegistrationManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppRegistrationManager",
        "supportedByServerChanged",
        "",
        "passwordChanged",
        "newPassword",
        "passwordChangeFailed",
        "QXmppStanza::Error",
        "error",
        "registrationFormReceived",
        "QXmppRegisterIq",
        "iq",
        "accountDeleted",
        "accountDeletionFailed",
        "registrationSucceeded",
        "registrationFailed",
        "supportedByServer"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'supportedByServerChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'passwordChangeFailed'
        QtMocHelpers::SignalData<void(QXmppStanza::Error)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'registrationFormReceived'
        QtMocHelpers::SignalData<void(const QXmppRegisterIq &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'accountDeleted'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountDeletionFailed'
        QtMocHelpers::SignalData<void(QXmppStanza::Error)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'registrationSucceeded'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'registrationFailed'
        QtMocHelpers::SignalData<void(const QXmppStanza::Error &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'supportedByServer'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppRegistrationManager, qt_meta_tag_ZN24QXmppRegistrationManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppRegistrationManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppRegistrationManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppRegistrationManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QXmppRegistrationManagerE_t>.metaTypes,
    nullptr
} };

void QXmppRegistrationManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppRegistrationManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->supportedByServerChanged(); break;
        case 1: _t->passwordChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->passwordChangeFailed((*reinterpret_cast<std::add_pointer_t<QXmppStanza::Error>>(_a[1]))); break;
        case 3: _t->registrationFormReceived((*reinterpret_cast<std::add_pointer_t<QXmppRegisterIq>>(_a[1]))); break;
        case 4: _t->accountDeleted(); break;
        case 5: _t->accountDeletionFailed((*reinterpret_cast<std::add_pointer_t<QXmppStanza::Error>>(_a[1]))); break;
        case 6: _t->registrationSucceeded(); break;
        case 7: _t->registrationFailed((*reinterpret_cast<std::add_pointer_t<QXmppStanza::Error>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)()>(_a, &QXmppRegistrationManager::supportedByServerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)(const QString & )>(_a, &QXmppRegistrationManager::passwordChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)(QXmppStanza::Error )>(_a, &QXmppRegistrationManager::passwordChangeFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)(const QXmppRegisterIq & )>(_a, &QXmppRegistrationManager::registrationFormReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)()>(_a, &QXmppRegistrationManager::accountDeleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)(QXmppStanza::Error )>(_a, &QXmppRegistrationManager::accountDeletionFailed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)()>(_a, &QXmppRegistrationManager::registrationSucceeded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppRegistrationManager::*)(const QXmppStanza::Error & )>(_a, &QXmppRegistrationManager::registrationFailed, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->supportedByServer(); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppRegistrationManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppRegistrationManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppRegistrationManagerE_t>.strings))
        return static_cast<void*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppRegistrationManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QXmppRegistrationManager::supportedByServerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppRegistrationManager::passwordChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QXmppRegistrationManager::passwordChangeFailed(QXmppStanza::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXmppRegistrationManager::registrationFormReceived(const QXmppRegisterIq & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QXmppRegistrationManager::accountDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QXmppRegistrationManager::accountDeletionFailed(QXmppStanza::Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QXmppRegistrationManager::registrationSucceeded()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QXmppRegistrationManager::registrationFailed(const QXmppStanza::Error & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
