/****************************************************************************
** Meta object code from reading C++ file 'QXmppMucManagerV2.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/client/QXmppMucManagerV2.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppMucManagerV2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QXmppMucManagerV2E_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppMucManagerV2::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppMucManagerV2E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppMucManagerV2",
        "participantJoined",
        "",
        "roomJid",
        "QXmppMucParticipant",
        "participant",
        "participantLeft",
        "QXmpp::Muc::LeaveReason",
        "reason",
        "removedFromRoom",
        "std::optional<QXmpp::Muc::Destroy>",
        "destroy",
        "roomHistoryReceived",
        "QList<QXmppMessage>",
        "messages",
        "messageReceived",
        "QXmppMessage",
        "message",
        "voiceRequestReceived",
        "QXmppMucVoiceRequest",
        "request",
        "invitationReceived",
        "QXmpp::Muc::Invite",
        "invite",
        "password",
        "invitationDeclined",
        "QXmpp::Muc::Decline",
        "decline",
        "directInvitationReceived",
        "QXmpp::Muc::DirectInvitation",
        "invitation",
        "from"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'participantJoined'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMucParticipant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'participantLeft'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMucParticipant &, QXmpp::Muc::LeaveReason)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'removedFromRoom'
        QtMocHelpers::SignalData<void(const QString &, QXmpp::Muc::LeaveReason, const std::optional<QXmpp::Muc::Destroy> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'roomHistoryReceived'
        QtMocHelpers::SignalData<void(const QString &, const QList<QXmppMessage> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMessage &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 16, 17 },
        }}),
        // Signal 'voiceRequestReceived'
        QtMocHelpers::SignalData<void(const QString &, const QXmppMucVoiceRequest &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 19, 20 },
        }}),
        // Signal 'invitationReceived'
        QtMocHelpers::SignalData<void(const QString &, const QXmpp::Muc::Invite &, const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 22, 23 }, { QMetaType::QString, 24 },
        }}),
        // Signal 'invitationDeclined'
        QtMocHelpers::SignalData<void(const QString &, const QXmpp::Muc::Decline &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 26, 27 },
        }}),
        // Signal 'directInvitationReceived'
        QtMocHelpers::SignalData<void(const QXmpp::Muc::DirectInvitation &, const QString &, const QXmppMessage &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 }, { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppMucManagerV2, qt_meta_tag_ZN17QXmppMucManagerV2E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppMucManagerV2::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppClientExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppMucManagerV2E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppMucManagerV2E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppMucManagerV2E_t>.metaTypes,
    nullptr
} };

void QXmppMucManagerV2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppMucManagerV2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->participantJoined((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMucParticipant>>(_a[2]))); break;
        case 1: _t->participantLeft((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMucParticipant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QXmpp::Muc::LeaveReason>>(_a[3]))); break;
        case 2: _t->removedFromRoom((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmpp::Muc::LeaveReason>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::optional<QXmpp::Muc::Destroy>>>(_a[3]))); break;
        case 3: _t->roomHistoryReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QXmppMessage>>>(_a[2]))); break;
        case 4: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[2]))); break;
        case 5: _t->voiceRequestReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmppMucVoiceRequest>>(_a[2]))); break;
        case 6: _t->invitationReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmpp::Muc::Invite>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 7: _t->invitationDeclined((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QXmpp::Muc::Decline>>(_a[2]))); break;
        case 8: _t->directInvitationReceived((*reinterpret_cast<std::add_pointer_t<QXmpp::Muc::DirectInvitation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QXmppMessage>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QXmppMessage> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMessage >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMucVoiceRequest >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QXmppMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmppMucParticipant & )>(_a, &QXmppMucManagerV2::participantJoined, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmppMucParticipant & , QXmpp::Muc::LeaveReason )>(_a, &QXmppMucManagerV2::participantLeft, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , QXmpp::Muc::LeaveReason , const std::optional<QXmpp::Muc::Destroy> & )>(_a, &QXmppMucManagerV2::removedFromRoom, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QList<QXmppMessage> & )>(_a, &QXmppMucManagerV2::roomHistoryReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmppMessage & )>(_a, &QXmppMucManagerV2::messageReceived, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmppMucVoiceRequest & )>(_a, &QXmppMucManagerV2::voiceRequestReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmpp::Muc::Invite & , const QString & )>(_a, &QXmppMucManagerV2::invitationReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QString & , const QXmpp::Muc::Decline & )>(_a, &QXmppMucManagerV2::invitationDeclined, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppMucManagerV2::*)(const QXmpp::Muc::DirectInvitation & , const QString & , const QXmppMessage & )>(_a, &QXmppMucManagerV2::directInvitationReceived, 8))
            return;
    }
}

const QMetaObject *QXmppMucManagerV2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppMucManagerV2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppMucManagerV2E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QXmppMessageHandler"))
        return static_cast< QXmppMessageHandler*>(this);
    return QXmppClientExtension::qt_metacast(_clname);
}

int QXmppMucManagerV2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppClientExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QXmppMucManagerV2::participantJoined(const QString & _t1, const QXmppMucParticipant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QXmppMucManagerV2::participantLeft(const QString & _t1, const QXmppMucParticipant & _t2, QXmpp::Muc::LeaveReason _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void QXmppMucManagerV2::removedFromRoom(const QString & _t1, QXmpp::Muc::LeaveReason _t2, const std::optional<QXmpp::Muc::Destroy> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QXmppMucManagerV2::roomHistoryReceived(const QString & _t1, const QList<QXmppMessage> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QXmppMucManagerV2::messageReceived(const QString & _t1, const QXmppMessage & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QXmppMucManagerV2::voiceRequestReceived(const QString & _t1, const QXmppMucVoiceRequest & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QXmppMucManagerV2::invitationReceived(const QString & _t1, const QXmpp::Muc::Invite & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void QXmppMucManagerV2::invitationDeclined(const QString & _t1, const QXmpp::Muc::Decline & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void QXmppMucManagerV2::directInvitationReceived(const QXmpp::Muc::DirectInvitation & _t1, const QString & _t2, const QXmppMessage & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
