/****************************************************************************
** Meta object code from reading C++ file 'QXmppLogger.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/base/QXmppLogger.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppLogger.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QXmppLoggerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppLogger::qt_create_metaobjectdata<qt_meta_tag_ZN11QXmppLoggerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppLogger",
        "loggingTypeChanged",
        "",
        "logFilePathChanged",
        "messageTypesChanged",
        "prettyXmlChanged",
        "colorModeChanged",
        "message",
        "QXmppLogger::MessageType",
        "type",
        "text",
        "setGauge",
        "gauge",
        "value",
        "updateCounter",
        "counter",
        "amount",
        "log",
        "reopen",
        "logFilePath",
        "loggingType",
        "LoggingType",
        "messageTypes",
        "MessageTypes",
        "prettyXml",
        "colorMode",
        "ColorMode",
        "NoLogging",
        "FileLogging",
        "StdoutLogging",
        "SignalLogging",
        "MessageType",
        "NoMessage",
        "DebugMessage",
        "InformationMessage",
        "WarningMessage",
        "ReceivedMessage",
        "SentMessage",
        "AnyMessage",
        "ColorOff",
        "ColorOn",
        "ColorAuto"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loggingTypeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'logFilePathChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'messageTypesChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'prettyXmlChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorModeChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'message'
        QtMocHelpers::SignalData<void(QXmppLogger::MessageType, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'setGauge'
        QtMocHelpers::SlotData<void(const QString &, double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'updateCounter'
        QtMocHelpers::SlotData<void(const QString &, qint64)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::LongLong, 16 },
        }}),
        // Slot 'log'
        QtMocHelpers::SlotData<void(QXmppLogger::MessageType, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'reopen'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'logFilePath'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'loggingType'
        QtMocHelpers::PropertyData<enum LoggingType>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'messageTypes'
        QtMocHelpers::PropertyData<MessageTypes>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'prettyXml'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'colorMode'
        QtMocHelpers::PropertyData<enum ColorMode>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LoggingType'
        QtMocHelpers::EnumData<enum LoggingType>(21, 21, QMC::EnumFlags{}).add({
            {   27, LoggingType::NoLogging },
            {   28, LoggingType::FileLogging },
            {   29, LoggingType::StdoutLogging },
            {   30, LoggingType::SignalLogging },
        }),
        // enum 'MessageType'
        QtMocHelpers::EnumData<enum MessageType>(31, 31, QMC::EnumFlags{}).add({
            {   32, MessageType::NoMessage },
            {   33, MessageType::DebugMessage },
            {   34, MessageType::InformationMessage },
            {   35, MessageType::WarningMessage },
            {   36, MessageType::ReceivedMessage },
            {   37, MessageType::SentMessage },
            {   38, MessageType::AnyMessage },
        }),
        // flag 'MessageTypes'
        QtMocHelpers::EnumData<MessageTypes>(23, 31, QMC::EnumIsFlag).add({
            {   32, MessageType::NoMessage },
            {   33, MessageType::DebugMessage },
            {   34, MessageType::InformationMessage },
            {   35, MessageType::WarningMessage },
            {   36, MessageType::ReceivedMessage },
            {   37, MessageType::SentMessage },
            {   38, MessageType::AnyMessage },
        }),
        // enum 'ColorMode'
        QtMocHelpers::EnumData<enum ColorMode>(26, 26, QMC::EnumFlags{}).add({
            {   39, ColorMode::ColorOff },
            {   40, ColorMode::ColorOn },
            {   41, ColorMode::ColorAuto },
        }),
    };
    return QtMocHelpers::metaObjectData<QXmppLogger, qt_meta_tag_ZN11QXmppLoggerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppLogger::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppLoggerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppLoggerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QXmppLoggerE_t>.metaTypes,
    nullptr
} };

void QXmppLogger::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppLogger *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loggingTypeChanged(); break;
        case 1: _t->logFilePathChanged(); break;
        case 2: _t->messageTypesChanged(); break;
        case 3: _t->prettyXmlChanged(); break;
        case 4: _t->colorModeChanged(); break;
        case 5: _t->message((*reinterpret_cast<std::add_pointer_t<QXmppLogger::MessageType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->setGauge((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 7: _t->updateCounter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 8: _t->log((*reinterpret_cast<std::add_pointer_t<QXmppLogger::MessageType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->reopen(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)()>(_a, &QXmppLogger::loggingTypeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)()>(_a, &QXmppLogger::logFilePathChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)()>(_a, &QXmppLogger::messageTypesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)()>(_a, &QXmppLogger::prettyXmlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)()>(_a, &QXmppLogger::colorModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLogger::*)(QXmppLogger::MessageType , const QString & )>(_a, &QXmppLogger::message, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->logFilePath(); break;
        case 1: *reinterpret_cast<enum LoggingType*>(_v) = _t->loggingType(); break;
        case 2: QtMocHelpers::assignFlags<MessageTypes>(_v, _t->messageTypes()); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->prettyXml(); break;
        case 4: *reinterpret_cast<enum ColorMode*>(_v) = _t->colorMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLogFilePath(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLoggingType(*reinterpret_cast<enum LoggingType*>(_v)); break;
        case 2: _t->setMessageTypes(*reinterpret_cast<MessageTypes*>(_v)); break;
        case 3: _t->setPrettyXml(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setColorMode(*reinterpret_cast<enum ColorMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXmppLogger::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppLogger::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QXmppLoggerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QXmppLogger::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QXmppLogger::loggingTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QXmppLogger::logFilePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QXmppLogger::messageTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QXmppLogger::prettyXmlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QXmppLogger::colorModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QXmppLogger::message(QXmppLogger::MessageType _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN13QXmppLoggableE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppLoggable::qt_create_metaobjectdata<qt_meta_tag_ZN13QXmppLoggableE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppLoggable",
        "setGauge",
        "",
        "gauge",
        "value",
        "logMessage",
        "QXmppLogger::MessageType",
        "type",
        "msg",
        "updateCounter",
        "counter",
        "amount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setGauge'
        QtMocHelpers::SignalData<void(const QString &, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 4 },
        }}),
        // Signal 'logMessage'
        QtMocHelpers::SignalData<void(QXmppLogger::MessageType, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'updateCounter'
        QtMocHelpers::SignalData<void(const QString &, qint64)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::LongLong, 11 },
        }}),
        // Signal 'updateCounter'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppLoggable, qt_meta_tag_ZN13QXmppLoggableE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppLoggable::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QXmppLoggableE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QXmppLoggableE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QXmppLoggableE_t>.metaTypes,
    nullptr
} };

void QXmppLoggable::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppLoggable *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setGauge((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: _t->logMessage((*reinterpret_cast<std::add_pointer_t<QXmppLogger::MessageType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->updateCounter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 3: _t->updateCounter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXmppLoggable::*)(const QString & , double )>(_a, &QXmppLoggable::setGauge, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLoggable::*)(QXmppLogger::MessageType , const QString & )>(_a, &QXmppLoggable::logMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXmppLoggable::*)(const QString & , qint64 )>(_a, &QXmppLoggable::updateCounter, 2))
            return;
    }
}

const QMetaObject *QXmppLoggable::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppLoggable::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QXmppLoggableE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QXmppLoggable::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QXmppLoggable::setGauge(const QString & _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QXmppLoggable::logMessage(QXmppLogger::MessageType _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QXmppLoggable::updateCounter(const QString & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
