/****************************************************************************
** Meta object code from reading C++ file 'QXmppSasl_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qxmpp/src/base/QXmppSasl_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QXmppSasl_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QXmppSaslClientE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClient::qt_create_metaobjectdata<qt_meta_tag_ZN15QXmppSaslClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClient"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClient, qt_meta_tag_ZN15QXmppSaslClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QXmppSaslClientE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClient *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslClientE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppSaslClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15QXmppSaslServerE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslServer::qt_create_metaobjectdata<qt_meta_tag_ZN15QXmppSaslServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslServer"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslServer, qt_meta_tag_ZN15QXmppSaslServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppLoggable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QXmppSaslServerE_t>.metaTypes,
    nullptr
} };

void QXmppSaslServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslServer *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QXmppSaslServerE_t>.strings))
        return static_cast<void*>(this);
    return QXmppLoggable::qt_metacast(_clname);
}

int QXmppSaslServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppLoggable::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientAnonymous::qt_create_metaobjectdata<qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientAnonymous"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientAnonymous, qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientAnonymous::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientAnonymous::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientAnonymous *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientAnonymous::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientAnonymous::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientAnonymousE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientAnonymous::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientDigestMd5::qt_create_metaobjectdata<qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientDigestMd5"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientDigestMd5, qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientDigestMd5::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientDigestMd5::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientDigestMd5 *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientDigestMd5::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientDigestMd5::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslClientDigestMd5E_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientDigestMd5::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23QXmppSaslClientFacebookE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientFacebook::qt_create_metaobjectdata<qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientFacebook"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientFacebook, qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientFacebook::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientFacebook::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientFacebook *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientFacebook::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientFacebook::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QXmppSaslClientFacebookE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientFacebook::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN21QXmppSaslClientGoogleE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientGoogle::qt_create_metaobjectdata<qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientGoogle"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientGoogle, qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientGoogle::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientGoogle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientGoogle *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientGoogle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientGoogle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QXmppSaslClientGoogleE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientGoogle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20QXmppSaslClientPlainE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientPlain::qt_create_metaobjectdata<qt_meta_tag_ZN20QXmppSaslClientPlainE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientPlain"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientPlain, qt_meta_tag_ZN20QXmppSaslClientPlainE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientPlain::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientPlainE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientPlainE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QXmppSaslClientPlainE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientPlain::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientPlain *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientPlain::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientPlain::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientPlainE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientPlain::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20QXmppSaslClientScramE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientScram::qt_create_metaobjectdata<qt_meta_tag_ZN20QXmppSaslClientScramE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientScram"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientScram, qt_meta_tag_ZN20QXmppSaslClientScramE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientScram::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientScramE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientScramE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QXmppSaslClientScramE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientScram::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientScram *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientScram::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientScram::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslClientScramE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientScram::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QXmppSaslClientHtE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientHt::qt_create_metaobjectdata<qt_meta_tag_ZN17QXmppSaslClientHtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientHt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientHt, qt_meta_tag_ZN17QXmppSaslClientHtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientHt::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppSaslClientHtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppSaslClientHtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QXmppSaslClientHtE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientHt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientHt *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientHt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientHt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QXmppSaslClientHtE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientHt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslClientWindowsLive::qt_create_metaobjectdata<qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslClientWindowsLive"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslClientWindowsLive, qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslClientWindowsLive::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslClient::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>.metaTypes,
    nullptr
} };

void QXmppSaslClientWindowsLive::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslClientWindowsLive *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslClientWindowsLive::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslClientWindowsLive::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QXmppSaslClientWindowsLiveE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslClient::qt_metacast(_clname);
}

int QXmppSaslClientWindowsLive::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslClient::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslServerAnonymous::qt_create_metaobjectdata<qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslServerAnonymous"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslServerAnonymous, qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslServerAnonymous::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslServer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>.metaTypes,
    nullptr
} };

void QXmppSaslServerAnonymous::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslServerAnonymous *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslServerAnonymous::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslServerAnonymous::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerAnonymousE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslServer::qt_metacast(_clname);
}

int QXmppSaslServerAnonymous::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslServer::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslServerDigestMd5::qt_create_metaobjectdata<qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslServerDigestMd5"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslServerDigestMd5, qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslServerDigestMd5::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslServer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>.metaTypes,
    nullptr
} };

void QXmppSaslServerDigestMd5::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslServerDigestMd5 *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslServerDigestMd5::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslServerDigestMd5::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QXmppSaslServerDigestMd5E_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslServer::qt_metacast(_clname);
}

int QXmppSaslServerDigestMd5::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslServer::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20QXmppSaslServerPlainE_t {};
} // unnamed namespace

template <> constexpr inline auto QXmppSaslServerPlain::qt_create_metaobjectdata<qt_meta_tag_ZN20QXmppSaslServerPlainE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXmppSaslServerPlain"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QXmppSaslServerPlain, qt_meta_tag_ZN20QXmppSaslServerPlainE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXmppSaslServerPlain::staticMetaObject = { {
    QMetaObject::SuperData::link<QXmppSaslServer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslServerPlainE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslServerPlainE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QXmppSaslServerPlainE_t>.metaTypes,
    nullptr
} };

void QXmppSaslServerPlain::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXmppSaslServerPlain *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QXmppSaslServerPlain::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXmppSaslServerPlain::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QXmppSaslServerPlainE_t>.strings))
        return static_cast<void*>(this);
    return QXmppSaslServer::qt_metacast(_clname);
}

int QXmppSaslServerPlain::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXmppSaslServer::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
